@@ -1,5 +1,5 @@
 Clazz.declarePackage ("J.adapter.readers.pymol");
-Clazz.load (["J.adapter.readers.pdb.PdbReader", "J.api.PymolAtomReader", "JU.BS", "$.P3"], "J.adapter.readers.pymol.PyMOLReader", ["java.lang.Boolean", "java.util.Hashtable", "JU.Lst", "$.PT", "$.V3", "J.adapter.readers.pymol.PickleReader", "$.PyMOL", "$.PyMOLScene", "J.adapter.smarter.Atom", "$.Bond", "$.Structure", "J.c.STR", "JU.BSUtil", "$.BoxInfo", "$.Logger"], function () {
+Clazz.load (["J.adapter.readers.pdb.PdbReader", "J.api.PymolAtomReader", "JU.BS", "$.P3"], "J.adapter.readers.pymol.PyMOLReader", ["java.lang.Boolean", "java.util.Hashtable", "JU.AU", "$.BC", "$.CU", "$.Lst", "$.PT", "$.V3", "J.adapter.readers.pymol.PickleReader", "$.PyMOL", "$.PyMOLScene", "J.adapter.smarter.Atom", "$.Bond", "$.Structure", "J.c.STR", "JU.BSUtil", "$.BoxInfo", "$.Logger"], function () {
 c$ = Clazz.decorateAsClass (function () {
 this.allowSurface = true;
 this.doResize = false;
@@ -39,6 +39,9 @@ this.haveScenes = false;
 this.baseModelIndex = 0;
 this.sceneOrder = null;
 this.bondCount = 0;
+this.haveBinaryArrays = true;
+this.ptTemp = null;
+this.aTemp = null;
 Clazz.instantialize (this, arguments);
 }, J.adapter.readers.pymol, "PyMOLReader", J.adapter.readers.pdb.PdbReader, J.api.PymolAtomReader);
 Clazz.prepareFields (c$, function () {
@@ -46,6 +49,8 @@ this.bsStructureDefined =  new JU.BS ();
 this.xyzMin = JU.P3.new3 (1e6, 1e6, 1e6);
 this.xyzMax = JU.P3.new3 (-1000000.0, -1000000.0, -1000000.0);
 this.reps =  new Array (23);
+this.ptTemp =  new JU.P3 ();
+this.aTemp =  Clazz.newByteArray (16, 0);
 });
 Clazz.overrideMethod (c$, "setup", 
 function (fullPath, htParams, reader) {
@@ -96,7 +101,7 @@ this.appendLoadNote (this.vwr.getMeasurementInfoAsString ());
 this.setLoadNote ();
 }if (this.haveScenes) {
 var scenes =  new Array (this.sceneOrder.size ());
-for (var i = scenes.length; --i >= 0; ) scenes[i] = this.sceneOrder.get (i);
+for (var i = scenes.length; --i >= 0; ) scenes[i] = J.adapter.readers.pymol.PyMOLReader.stringAt (this.sceneOrder, i);
 
 this.vwr.ms.msInfo.put ("scenes", scenes);
 }this.vwr.ms.setTrajectoryBs (JU.BSUtil.newBitSet2 (this.baseModelIndex, this.vwr.ms.mc));
@@ -116,10 +121,12 @@ Clazz.defineMethod (c$, "process",
 this.pymolVersion = (map.get ("version")).intValue ();
 this.appendLoadNote ("PyMOL version: " + this.pymolVersion);
 var settings = this.fixSettings (J.adapter.readers.pymol.PyMOLReader.getMapList (map, "settings"));
+var lst = J.adapter.readers.pymol.PyMOLReader.listAt (settings, 749);
+this.haveBinaryArrays = (lst != null && J.adapter.readers.pymol.PyMOLReader.floatAt (lst, 2) == 1);
 this.sceneOrder = J.adapter.readers.pymol.PyMOLReader.getMapList (map, "scene_order");
 this.haveScenes = this.getFrameScenes (map);
-var file = this.listAt (settings, 440);
-if (file != null) JU.Logger.info ("PyMOL session file: " + file.get (2));
+var file = J.adapter.readers.pymol.PyMOLReader.listAt (settings, 440);
+if (file != null && file.size () > 2) JU.Logger.info ("PyMOL session file: " + file.get (2));
 this.setUniqueSettings (J.adapter.readers.pymol.PyMOLReader.getMapList (map, "unique_settings"));
 this.pymolScene =  new J.adapter.readers.pymol.PyMOLScene (this, this.vwr, settings, this.uniqueSettings, this.pymolVersion, this.haveScenes, this.baseAtomIndex, this.baseModelIndex, this.doCache, this.filePath);
 var logFile = this.vwr.getLogFileName ();
@@ -131,7 +138,7 @@ var name = e.getKey ();
 JU.Logger.info (name);
 if (name.equals ("names")) {
 for (var i = 1; i < names.size (); i++) {
-var obj = this.listAt (names, i);
+var obj = J.adapter.readers.pymol.PyMOLReader.listAt (names, i);
 JU.Logger.info ("  " + J.adapter.readers.pymol.PyMOLReader.stringAt (obj, 0));
 }
 }}
@@ -147,7 +154,7 @@ this.vwr.log ("\n===names===");
 for (var i = 1; i < names.size (); i++) {
 this.vwr.log ("");
 var list = names.get (i);
-this.vwr.log (" =" + list.get (0).toString () + "=");
+this.vwr.log (" =" + J.adapter.readers.pymol.PyMOLReader.bytesToString (list.get (0)) + "=");
 try {
 this.vwr.log (JU.PT.rep (list.toString (), "[", "\n["));
 } catch (e) {
@@ -203,10 +210,10 @@ if (this.isTrajectory) {
 this.trajectoryStep =  new Array (this.totalAtomCount);
 this.trajectorySteps.addLast (this.trajectoryStep);
 this.trajectoryPtr = 0;
-}for (var i = 1; i < n; i++) this.processObject (this.listAt (names, i), true, j);
+}for (var i = 1; i < n; i++) this.processObject (J.adapter.readers.pymol.PyMOLReader.listAt (names, i), true, j);
 
 }
-for (var i = 1; i < n; i++) this.processObject (this.listAt (names, i), false, 0);
+for (var i = 1; i < n; i++) this.processObject (J.adapter.readers.pymol.PyMOLReader.listAt (names, i), false, 0);
 
 this.pymolScene.setReaderObjectInfo (null, 0, null, false, null, null, null);
 if (this.mapObjects != null && this.allowSurface) this.processMeshes ();
@@ -217,7 +224,7 @@ this.asc.finalizeTrajectoryAs (this.trajectorySteps, null);
 this.processSelectionsAndScenes (map);
 this.pymolScene.finalizeVisibility ();
 if (!this.isStateScript) {
-this.vwr.initialize (false);
+this.vwr.initialize (false, true);
 this.addJmolScript (this.pymolScene.getViewScript (J.adapter.readers.pymol.PyMOLReader.getMapList (map, "view")).toString ());
 }if (this.$ac == 0) this.asc.setInfo ("dataOnly", Boolean.TRUE);
 this.pymolScene.offsetObjects ();
@@ -246,7 +253,7 @@ var scenes = map.get ("scene_dict");
 for (var i = 0; i < this.sceneOrder.size (); i++) {
 var name = J.adapter.readers.pymol.PyMOLReader.stringAt (this.sceneOrder, i);
 var thisScene = J.adapter.readers.pymol.PyMOLReader.getMapList (scenes, name);
-if (thisScene == null || thisScene.get (2) == null) this.sceneOrder.remove (i--);
+if (thisScene == null || thisScene.get (2) == null) this.sceneOrder.removeItemAt (i--);
 }
 return (this.sceneOrder != null && this.sceneOrder.size () != 0);
 }, "java.util.Map");
@@ -272,26 +279,35 @@ Clazz.defineMethod (c$, "addColors",
  function (colors, isClamped) {
 if (colors == null || colors.size () == 0) return;
 for (var i = colors.size (); --i >= 0; ) {
-var c = this.listAt (colors, i);
-J.adapter.readers.pymol.PyMOL.addColor (c.get (1), isClamped ? J.adapter.readers.pymol.PyMOLScene.colorSettingClamped (c) : J.adapter.readers.pymol.PyMOLScene.getColorPt (c.get (2)));
+var c = J.adapter.readers.pymol.PyMOLReader.listAt (colors, i);
+J.adapter.readers.pymol.PyMOL.addColor (c.get (1), isClamped ? J.adapter.readers.pymol.PyMOLReader.colorSettingClamped (c, this.ptTemp) : J.adapter.readers.pymol.PyMOLReader.getColorPt (c.get (2), this.ptTemp));
 }
 }, "JU.Lst,~B");
 Clazz.defineMethod (c$, "getAtomAndStateCount", 
  function (names) {
 var n = 0;
 for (var i = 1; i < names.size (); i++) {
-var execObject = this.listAt (names, i);
+var execObject = J.adapter.readers.pymol.PyMOLReader.listAt (names, i);
 var type = J.adapter.readers.pymol.PyMOLReader.intAt (execObject, 4);
 if (!this.checkObject (execObject)) continue;
 if (type == 1) {
-var pymolObject = this.listAt (execObject, 5);
-var states = this.listAt (pymolObject, 4);
+var pymolObject = J.adapter.readers.pymol.PyMOLReader.listAt (execObject, 5);
+var states = J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 4);
 var ns = states.size ();
 if (ns > this.stateCount) this.stateCount = ns;
-var nAtoms = this.listAt (pymolObject, 7).size ();
+var nAtoms;
+var nBonds;
+if (this.haveBinaryArrays) {
+nBonds = Clazz.doubleToInt ((J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 6).get (1)).length / 20);
+nAtoms = Clazz.doubleToInt ((J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 7).get (1)).length / 120);
+n += nAtoms;
+} else {
+nBonds = J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 6).size ();
+nAtoms = J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 7).size ();
+}System.out.println ("Object " + this.objectName + " nBonds=" + nBonds + ", nAtoms = " + nAtoms);
 for (var j = 0; j < ns; j++) {
-var state = this.listAt (states, j);
-var idxToAtm = this.listAt (state, 3);
+var state = J.adapter.readers.pymol.PyMOLReader.listAt (states, j);
+var idxToAtm = J.adapter.readers.pymol.PyMOLReader.listAt (state, 3);
 if (idxToAtm == null) {
 this.isTrajectory = false;
 } else {
@@ -318,7 +334,7 @@ movie.put ("currentFrame", Integer.$valueOf (this.pymolFrame - 1));
 var haveCommands = false;
 var haveViews = false;
 var haveFrames = false;
-var list = this.listAt (mov, 4);
+var list = J.adapter.readers.pymol.PyMOLReader.listAt (mov, 4);
 for (var i = list.size (); --i >= 0; ) if (J.adapter.readers.pymol.PyMOLReader.intAt (list, i) != 0) {
 this.frames =  Clazz.newIntArray (list.size (), 0);
 for (var j = this.frames.length; --j >= 0; ) this.frames[j] = J.adapter.readers.pymol.PyMOLReader.intAt (list, j) + 1;
@@ -327,7 +343,7 @@ movie.put ("frames", this.frames);
 haveFrames = true;
 break;
 }
-var cmds = this.listAt (mov, 5);
+var cmds = J.adapter.readers.pymol.PyMOLReader.listAt (mov, 5);
 var cmd;
 for (var i = cmds.size (); --i >= 0; ) if ((cmd = J.adapter.readers.pymol.PyMOLReader.stringAt (cmds, i)) != null && cmd.length > 1) {
 cmds = J.adapter.readers.pymol.PyMOLReader.fixMovieCommands (cmds);
@@ -336,9 +352,9 @@ movie.put ("commands", cmds);
 haveCommands = true;
 break;
 }}
-var views = this.listAt (mov, 6);
+var views = J.adapter.readers.pymol.PyMOLReader.listAt (mov, 6);
 var view;
-for (var i = views.size (); --i >= 0; ) if ((view = this.listAt (views, i)) != null && view.size () >= 12 && view.get (1) != null) {
+for (var i = views.size (); --i >= 0; ) if ((view = J.adapter.readers.pymol.PyMOLReader.listAt (views, i)) != null && view.size () >= 12 && view.get (1) != null) {
 haveViews = true;
 views = J.adapter.readers.pymol.PyMOLReader.fixMovieViews (views);
 if (views != null) {
@@ -366,14 +382,14 @@ if (execObject == null) return;
 var type = J.adapter.readers.pymol.PyMOLReader.intAt (execObject, 4);
 var startLen = execObject.get (execObject.size () - 1);
 if ((type == 1) != moleculeOnly || !this.checkObject (execObject)) return;
-var pymolObject = this.listAt (execObject, 5);
+var pymolObject = J.adapter.readers.pymol.PyMOLReader.listAt (execObject, 5);
 var stateSettings = null;
 if (type == 1) {
-var states = this.listAt (pymolObject, 4);
-var state = this.listAt (states, iState);
-var idxToAtm = this.listAt (state, 3);
+var states = J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 4);
+var state = J.adapter.readers.pymol.PyMOLReader.listAt (states, iState);
+var idxToAtm = J.adapter.readers.pymol.PyMOLReader.listAt (state, 3);
 if (iState > 0 && (idxToAtm == null || idxToAtm.size () == 0)) return;
-stateSettings = this.listAt (state, 7);
+stateSettings = J.adapter.readers.pymol.PyMOLReader.listAt (state, 7);
 } else if (iState > 0) {
 return;
 }JU.Logger.info ("PyMOL model " + (this.nModels) + " Object " + this.objectName + (this.isHidden ? " (hidden)" : " (visible)"));
@@ -382,10 +398,10 @@ if (this.pymolFrame > 0 && this.pymolFrame != this.nModels) {
 this.pymolFrame = this.nModels;
 this.allStates = true;
 this.pymolScene.setFrameObject (4115, Integer.$valueOf (-1));
-}}var objectHeader = this.listAt (pymolObject, 0);
+}}var objectHeader = J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 0);
 var parentGroupName = (execObject.size () < 8 ? null : J.adapter.readers.pymol.PyMOLReader.stringAt (execObject, 6));
 if (" ".equals (parentGroupName)) parentGroupName = null;
-this.pymolScene.setReaderObjectInfo (this.objectName, type, parentGroupName, this.isHidden, this.listAt (objectHeader, 8), stateSettings, (moleculeOnly ? "_" + (iState + 1) : ""));
+this.pymolScene.setReaderObjectInfo (this.objectName, type, parentGroupName, this.isHidden, J.adapter.readers.pymol.PyMOLReader.listAt (objectHeader, 8), stateSettings, (moleculeOnly ? "_" + (iState + 1) : ""));
 var bsAtoms = null;
 var doExclude = (this.bsBytesExcluded != null);
 var msg = null;
@@ -446,8 +462,8 @@ Clazz.defineMethod (c$, "processCGO",
  function (pymolObject) {
 if (this.isStateScript) return;
 if (this.isHidden) return;
-var data = this.listAt (this.listAt (pymolObject, 2), 0);
-var color = J.adapter.readers.pymol.PyMOL.getRGB (J.adapter.readers.pymol.PyMOLReader.intAt (this.listAt (pymolObject, 0), 2));
+var data = J.adapter.readers.pymol.PyMOLReader.sublistAt (pymolObject, [2, 0]);
+var color = J.adapter.readers.pymol.PyMOL.getRGB (J.adapter.readers.pymol.PyMOLReader.intAt (J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 0), 2));
 var name = this.pymolScene.addCGO (data, color);
 if (name != null) this.appendLoadNote ("CGO " + name);
 }, "JU.Lst");
@@ -473,42 +489,70 @@ Clazz.defineMethod (c$, "processMeasure",
 if (this.isStateScript) return;
 if (this.isHidden) return;
 JU.Logger.info ("PyMOL measure " + this.objectName);
-var measure = this.listAt (this.listAt (pymolObject, 2), 0);
+var measure = J.adapter.readers.pymol.PyMOLReader.sublistAt (pymolObject, [2, 0]);
 var pt;
 var nCoord = (Clazz.instanceOf (measure.get (pt = 1), JU.Lst) ? 2 : Clazz.instanceOf (measure.get (pt = 4), JU.Lst) ? 3 : Clazz.instanceOf (measure.get (pt = 6), JU.Lst) ? 4 : 0);
 if (nCoord == 0) return;
-var setting = this.listAt (pymolObject, 0);
-var bsReps = J.adapter.readers.pymol.PyMOLReader.getBsReps (this.listAt (setting, 3));
-var list = this.listAt (measure, pt);
-var offsets = this.listAt (measure, 8);
+var setting = J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 0);
+var bsReps = J.adapter.readers.pymol.PyMOLReader.getBsReps (J.adapter.readers.pymol.PyMOLReader.listAt (setting, 3));
+var list = J.adapter.readers.pymol.PyMOLReader.listAt (measure, pt);
+var offsets = J.adapter.readers.pymol.PyMOLReader.listAt (measure, 8);
 var haveLabels = (measure.size () > 8);
 var color = J.adapter.readers.pymol.PyMOLReader.intAt (setting, 2);
 if (this.pymolScene.addMeasurements (null, nCoord, list, bsReps, color, offsets, haveLabels)) this.haveMeasurements = true;
 }, "JU.Lst");
 Clazz.defineMethod (c$, "processMolecule", 
  function (pymolObject, iState) {
-var states = this.listAt (pymolObject, 4);
-var state = this.listAt (states, iState);
-var idxToAtm = this.listAt (state, 3);
-var n = (idxToAtm == null ? 0 : idxToAtm.size ());
-if (n == 0) return null;
-this.$ac = this.ac0 = this.asc.ac;
+var states = J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 4);
+var state = J.adapter.readers.pymol.PyMOLReader.listAt (states, iState);
+var idxToAtm;
+var coords;
+var labelPositions;
+var idxArray = null;
+var coordsArray = null;
+var labelArray = null;
+var nBonds = J.adapter.readers.pymol.PyMOLReader.intAt (pymolObject, 2);
 var nAtoms = J.adapter.readers.pymol.PyMOLReader.intAt (pymolObject, 3);
+var n = nAtoms;
+if (this.haveBinaryArrays && JU.AU.isAB (state.get (3))) {
+idxToAtm = coords = labelPositions = null;
+idxArray =  Clazz.newIntArray (nAtoms, 0);
+coordsArray =  Clazz.newFloatArray (nAtoms * 3, 0);
+this.fillFloatArrayFromBytes (state.get (2), coordsArray);
+this.fillIntArrayFromBytes (state.get (3), idxArray);
+var b = state.get (8);
+if (b != null) {
+labelArray =  Clazz.newFloatArray (nAtoms * 7, 0);
+this.fillFloatArrayFromBytes (b, labelArray);
+}} else {
+coords = J.adapter.readers.pymol.PyMOLReader.listAt (state, 2);
+idxToAtm = J.adapter.readers.pymol.PyMOLReader.listAt (state, 3);
+labelPositions = J.adapter.readers.pymol.PyMOLReader.listAt (state, 8);
+if (idxToAtm != null) n = idxToAtm.size ();
+}if (n == 0) return null;
+this.$ac = this.ac0 = this.asc.ac;
 if (nAtoms == 0) return null;
 this.ssMapSeq =  new java.util.Hashtable ();
-if (iState == 0) this.processMolCryst (this.listAt (pymolObject, 10));
-var bonds = this.getBondList (this.listAt (pymolObject, 6));
-var pymolAtoms = this.listAt (pymolObject, 7);
+if (iState == 0) this.processMolCryst (J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 10));
+var bonds = this.getBondList (J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 6));
+var pymolAtoms = J.adapter.readers.pymol.PyMOLReader.listAt (pymolObject, 7);
 this.atomMap =  Clazz.newIntArray (nAtoms, 0);
 var bsAtoms = this.pymolScene.setAtomMap (this.atomMap, this.ac0);
 for (var i = 0; i < 23; i++) this.reps[i] = JU.BS.newN (1000);
 
-var coords = this.listAt (state, 2);
-var labelPositions = this.listAt (state, 8);
 if (iState == 0 || !this.isTrajectory) {
 this.pymolScene.ensureCapacity (n);
-for (var idx = 0; idx < n; idx++) {
-var a = this.addAtom (pymolAtoms, J.adapter.readers.pymol.PyMOLReader.intAt (idxToAtm, idx), idx, coords, labelPositions, bsAtoms, iState);
+var lexStr = null;
+var atomArray = null;
+var vArray = null;
+if (this.haveBinaryArrays) {
+var ver = J.adapter.readers.pymol.PyMOLReader.intAt (pymolAtoms, 0);
+atomArray = pymolAtoms.get (1);
+lexStr = this.getLexStr (pymolAtoms.get (2));
+System.out.println ("PyMOL atom dump version " + ver);
+vArray = (this.haveBinaryArrays ? J.adapter.readers.pymol.PyMOL.getVArray (ver) : null);
+}for (var idx = 0; idx < n; idx++) {
+var a = this.addAtom (pymolAtoms, (idxToAtm != null ? J.adapter.readers.pymol.PyMOLReader.intAt (idxToAtm, idx) : idxArray != null ? idxArray[idx] : idx), atomArray, vArray, lexStr, idx, coords, coordsArray, labelPositions, labelArray, bsAtoms, iState);
 if (a != null) this.trajectoryStep[this.trajectoryPtr++] = a;
 }
 }this.addBonds (bonds);
@@ -516,65 +560,231 @@ this.addMolStructures ();
 this.atoms = this.asc.atoms;
 if (!this.isStateScript) this.createShapeObjects ();
 this.ssMapSeq = null;
-JU.Logger.info ("reading " + (this.$ac - this.ac0) + " atoms");
+JU.Logger.info ("reading " + (this.$ac - this.ac0) + " atoms and " + nBonds + " bonds");
 JU.Logger.info ("----------");
 return bsAtoms;
 }, "JU.Lst,~N");
+Clazz.defineMethod (c$, "getLexStr", 
+ function (lex) {
+var pt = 0;
+var n = JU.BC.bytesToInt (lex, pt, false);
+var index =  Clazz.newIntArray (n, 0);
+var imax = 0;
+for (var i = 0; i < n; i++) {
+pt += 4;
+var ipt = index[i] = JU.BC.bytesToInt (lex, pt, false);
+if (ipt > imax) imax = ipt;
+}
+var tokens =  new Array (imax + 1);
+tokens[0] = " ";
+pt += 4;
+for (var i = 0; i < n; i++) {
+var s = tokens[index[i]] = this.getCStr (lex, pt);
+pt += s.length + 1;
+}
+return tokens;
+}, "~A");
+Clazz.defineMethod (c$, "getCStr", 
+ function (lex, pt) {
+try {
+var a = this.aTemp;
+var apt = 0;
+var b = 0;
+while ((b = lex[pt++]) != 0) {
+if (apt >= a.length) a = this.aTemp = JU.AU.doubleLengthByte (a);
+a[apt++] = b;
+}
+return  String.instantialize (JU.AU.arrayCopyByte (a, apt), "UTF-8");
+} catch (e) {
+if (Clazz.exceptionOf (e, java.io.UnsupportedEncodingException)) {
+return null;
+} else {
+throw e;
+}
+}
+}, "~A,~N");
 Clazz.defineMethod (c$, "processMolCryst", 
  function (cryst) {
 if (cryst == null || cryst.size () == 0) return;
-var l = this.listAt (this.listAt (cryst, 0), 0);
-var a = this.listAt (this.listAt (cryst, 0), 1);
-this.setUnitCell (this.floatAt (l, 0), this.floatAt (l, 1), this.floatAt (l, 2), this.floatAt (a, 0), this.floatAt (a, 1), this.floatAt (a, 2));
+var l = J.adapter.readers.pymol.PyMOLReader.sublistAt (cryst, [0, 0]);
+var a = J.adapter.readers.pymol.PyMOLReader.sublistAt (cryst, [0, 1]);
+this.setUnitCell (J.adapter.readers.pymol.PyMOLReader.floatAt (l, 0), J.adapter.readers.pymol.PyMOLReader.floatAt (l, 1), J.adapter.readers.pymol.PyMOLReader.floatAt (l, 2), J.adapter.readers.pymol.PyMOLReader.floatAt (a, 0), J.adapter.readers.pymol.PyMOLReader.floatAt (a, 1), J.adapter.readers.pymol.PyMOLReader.floatAt (a, 2));
 this.setSpaceGroupName (J.adapter.readers.pymol.PyMOLReader.stringAt (cryst, 1));
 }, "JU.Lst");
 Clazz.defineMethod (c$, "getBondList", 
  function (bonds) {
-var asSingle = this.pymolScene.booleanSetting (64);
+var asSingle = !this.pymolScene.booleanSetting (64);
+var b = null;
+var vArray = null;
 var n = bonds.size ();
-var bondList =  new JU.Lst ();
+var len = 0;
+if (this.haveBinaryArrays && n == 2) {
+var ver = J.adapter.readers.pymol.PyMOLReader.intAt (bonds, 0);
+System.out.println ("PyMOL bond dump version " + ver);
+vArray = J.adapter.readers.pymol.PyMOL.getVArrayB (ver);
+b = bonds.get (1);
+len = vArray[0];
+n = Clazz.doubleToInt (b.length / len);
+}var bondList =  new JU.Lst ();
 bondList.ensureCapacity (n);
-for (var i = 0; i < n; i++) {
-var b = this.listAt (bonds, i);
-var ia = J.adapter.readers.pymol.PyMOLReader.intAt (b, 0);
-var ib = J.adapter.readers.pymol.PyMOLReader.intAt (b, 1);
-var order = J.adapter.readers.pymol.PyMOLReader.intAt (b, 2);
-if (order < 1 || order > 3) order = 1;
+var ia;
+var ib;
+var order;
+var uid = -1;
+for (var i = 0, apt = 0; i < n; i++) {
+if (this.haveBinaryArrays) {
+ia = JU.BC.bytesToInt (b, apt + vArray[1], false);
+ib = JU.BC.bytesToInt (b, apt + vArray[2], false);
+uid = (b[apt + vArray[6]] == 0 ? -1 : JU.BC.bytesToInt (b, apt + vArray[5], false));
+order = b[apt + vArray[3]];
+apt += len;
+} else {
+var lst = J.adapter.readers.pymol.PyMOLReader.listAt (bonds, i);
+ia = J.adapter.readers.pymol.PyMOLReader.intAt (lst, 0);
+ib = J.adapter.readers.pymol.PyMOLReader.intAt (lst, 1);
+order = J.adapter.readers.pymol.PyMOLReader.intAt (lst, 2);
+uid = (lst.size () > 6 && J.adapter.readers.pymol.PyMOLReader.intAt (lst, 6) != 0 ? J.adapter.readers.pymol.PyMOLReader.intAt (lst, 5) : -1);
+}if (order < 1 || order > 3) order = 1;
 order |= (asSingle || order == 1 ? 65536 : 98304);
 var bond =  new J.adapter.smarter.Bond (ia, ib, order);
-bond.uniqueID = (b.size () > 6 && J.adapter.readers.pymol.PyMOLReader.intAt (b, 6) != 0 ? J.adapter.readers.pymol.PyMOLReader.intAt (b, 5) : -1);
+bond.uniqueID = uid;
 bondList.addLast (bond);
 }
 return bondList;
 }, "JU.Lst");
+Clazz.defineMethod (c$, "fillIntArrayFromBytes", 
+ function (b, array) {
+for (var i = 0, pt = 0; i < b.length; i += 4) array[pt++] = JU.BC.bytesToInt (b, i, false);
+
+}, "~A,~A");
+Clazz.defineMethod (c$, "fillFloatArrayFromBytes", 
+ function (b, array) {
+try {
+for (var i = 0, pt = 0; i < b.length; i += 4) array[pt++] = JU.BC.bytesToFloat (b, i, false);
+
+} catch (e) {
+if (Clazz.exceptionOf (e, Exception)) {
+} else {
+throw e;
+}
+}
+}, "~A,~A");
 Clazz.defineMethod (c$, "addAtom", 
- function (pymolAtoms, apt, icoord, coords, labelPositions, bsState, iState) {
+ function (pymolAtoms, apt, atomArray, vArray, lexStr, icoord, coords, coordArray, labelPositions, labelArray, bsState, iState) {
 this.atomMap[apt] = -1;
-var a = this.listAt (pymolAtoms, apt);
-var seqNo = J.adapter.readers.pymol.PyMOLReader.intAt (a, 0);
-var chainID = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 1);
-var altLoc = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 2);
-var insCode = " ";
-var name = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 6);
-var group3 = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 5);
+var chainID;
+var altLoc;
+var group3;
+var name;
+var sym;
+var label;
+var ssType;
+var resi;
+var insCode = null;
+var bfactor;
+var occupancy;
+var radius;
+var partialCharge;
+var seqNo;
+var intReps;
+var formalCharge;
+var atomColor;
+var serNo;
+var cartoonType;
+var flags;
+var uniqueID = -1;
+var isHetero;
+var bonded;
+var anisou = null;
+var bsReps = null;
+if (this.haveBinaryArrays) {
+var vpt;
+var pt = apt * vArray[0];
+seqNo = this.atomInt (atomArray, pt, vArray[1]);
+chainID = this.atomStr (atomArray, pt, vArray[34], lexStr);
+resi = this.atomStr (atomArray, pt, vArray[38], lexStr);
+group3 = this.atomStr (atomArray, pt, vArray[41], lexStr);
 if (group3.length > 3) group3 = group3.substring (0, 3);
+name = this.atomStr (atomArray, pt, vArray[36], lexStr);
+sym = this.atomStr (atomArray, pt, vArray[37], lexStr);
+label = this.atomStr (atomArray, pt, vArray[19], lexStr);
+ssType = this.atomStr (atomArray, pt, vArray[39], null);
+altLoc = this.atomStr (atomArray, pt, vArray[40], null);
+if ((vpt = vArray[42]) == 0) {
+resi = this.atomStr (atomArray, pt, vArray[38], null);
+} else {
+var b = atomArray[pt + vpt];
+insCode = (b == 0 ? " " : "" + String.fromCharCode (b));
+}bfactor = this.atomFloat (atomArray, pt, vArray[4]);
+occupancy = this.atomFloat (atomArray, pt, vArray[5]);
+radius = this.atomFloat (atomArray, pt, vArray[6]);
+partialCharge = this.atomFloat (atomArray, pt, vArray[7]);
+formalCharge = atomArray[pt + vArray[28]];
+if (formalCharge > 125) formalCharge -= 512;
+intReps = this.atomInt (atomArray, pt, vArray[20]);
+atomColor = this.atomInt (atomArray, pt, vArray[9]);
+serNo = this.atomInt (atomArray, pt, vArray[10]);
+cartoonType = this.atomInt (atomArray, pt, vArray[30]);
+flags = this.atomInt (atomArray, pt, vArray[11]);
+uniqueID = this.atomInt (atomArray, pt, vArray[13]);
+if (uniqueID == 0) uniqueID = -1;
+anisou =  Clazz.newFloatArray (8, 0);
+if ((vpt = vArray[45]) > 0) for (var i = 0; i < 6; i++) anisou[i] = JU.BC.bytesToShort (atomArray, pt + vpt + (i << 1), false);
+
+bonded = this.atomBool (atomArray, pt, vArray[22], vArray[47]);
+isHetero = this.atomBool (atomArray, pt, vArray[21], vArray[46]);
+} else {
+var a = J.adapter.readers.pymol.PyMOLReader.listAt (pymolAtoms, apt);
+seqNo = J.adapter.readers.pymol.PyMOLReader.intAt (a, 0);
+chainID = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 1);
+altLoc = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 2);
+resi = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 3);
+group3 = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 5);
+name = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 6);
+sym = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 7);
+label = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 9);
+ssType = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 10).substring (0, 1);
+bfactor = J.adapter.readers.pymol.PyMOLReader.floatAt (a, 14);
+occupancy = J.adapter.readers.pymol.PyMOLReader.floatAt (a, 15);
+radius = J.adapter.readers.pymol.PyMOLReader.floatAt (a, 16);
+partialCharge = J.adapter.readers.pymol.PyMOLReader.floatAt (a, 17);
+formalCharge = J.adapter.readers.pymol.PyMOLReader.intAt (a, 18);
+isHetero = (J.adapter.readers.pymol.PyMOLReader.intAt (a, 19) != 0);
+bsReps = J.adapter.readers.pymol.PyMOLReader.getBsReps (J.adapter.readers.pymol.PyMOLReader.listAt (a, 20));
+intReps = (bsReps == null ? J.adapter.readers.pymol.PyMOLReader.intAt (a, 20) : 0);
+atomColor = J.adapter.readers.pymol.PyMOLReader.intAt (a, 21);
+serNo = J.adapter.readers.pymol.PyMOLReader.intAt (a, 22);
+cartoonType = J.adapter.readers.pymol.PyMOLReader.intAt (a, 23);
+flags = J.adapter.readers.pymol.PyMOLReader.intAt (a, 24);
+bonded = (J.adapter.readers.pymol.PyMOLReader.intAt (a, 25) != 0);
+uniqueID = (a.size () > 40 && J.adapter.readers.pymol.PyMOLReader.intAt (a, 40) == 1 ? J.adapter.readers.pymol.PyMOLReader.intAt (a, 32) : -1);
+if (a.size () > 46) anisou = J.adapter.readers.pymol.PyMOLReader.floatsAt (a, 41,  Clazz.newFloatArray (8, 0), 6);
+}if (insCode == null) {
+var len = resi.length;
+var ch = (len > 0 ? resi.charAt (len - 1) : ' ');
+insCode = (JU.PT.isDigit (ch) ? " " : "" + ch);
+}if (group3.length > 3) group3 = group3.substring (0, 3);
 if (group3.equals (" ")) group3 = "UNK";
-var sym = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 7);
 if (sym.equals ("A")) sym = "C";
-var isHetero = (J.adapter.readers.pymol.PyMOLReader.intAt (a, 19) != 0);
 var ichain = this.vwr.getChainID (chainID, true);
 var atom = this.processAtom ( new J.adapter.smarter.Atom (), name, altLoc.charAt (0), group3, ichain, seqNo, insCode.charAt (0), isHetero, sym);
 if (!this.filterPDBAtom (atom, this.fileAtomIndex++)) return null;
+var x;
+var y;
+var z;
 icoord *= 3;
-var x = this.floatAt (coords, icoord);
-var y = this.floatAt (coords, ++icoord);
-var z = this.floatAt (coords, ++icoord);
-JU.BoxInfo.addPointXYZ (x, y, z, this.xyzMin, this.xyzMax, 0);
+if (this.haveBinaryArrays) {
+x = coordArray[icoord];
+y = coordArray[++icoord];
+z = coordArray[++icoord];
+} else {
+x = J.adapter.readers.pymol.PyMOLReader.floatAt (coords, icoord);
+y = J.adapter.readers.pymol.PyMOLReader.floatAt (coords, ++icoord);
+z = J.adapter.readers.pymol.PyMOLReader.floatAt (coords, ++icoord);
+}JU.BoxInfo.addPointXYZ (x, y, z, this.xyzMin, this.xyzMax, 0);
 if (this.isTrajectory && iState > 0) return null;
 var isNucleic = (J.adapter.readers.pymol.PyMOLReader.nucleic.indexOf (group3) >= 0);
 if (bsState != null) bsState.set (this.$ac);
-var label = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 9);
-var ssType = J.adapter.readers.pymol.PyMOLReader.stringAt (a, 10).substring (0, 1);
 if (seqNo >= -1000 && (!ssType.equals (" ") || name.equals ("CA") || isNucleic)) {
 var bs = this.ssMapSeq.get (ssType);
 if (bs == null) this.ssMapSeq.put (ssType, bs =  new JU.BS ());
@@ -583,31 +793,28 @@ ssType += ichain;
 bs = this.ssMapSeq.get (ssType);
 if (bs == null) this.ssMapSeq.put (ssType, bs =  new JU.BS ());
 bs.set (seqNo - -1000);
-}atom.bfactor = this.floatAt (a, 14);
-atom.foccupancy = this.floatAt (a, 15);
-atom.radius = this.floatAt (a, 16);
+}atom.bfactor = bfactor;
+atom.foccupancy = occupancy;
+atom.radius = radius;
 if (atom.radius == 0) atom.radius = 1;
-atom.partialCharge = this.floatAt (a, 17);
-var formalCharge = J.adapter.readers.pymol.PyMOLReader.intAt (a, 18);
-var bsReps = J.adapter.readers.pymol.PyMOLReader.getBsReps (this.listAt (a, 20));
-var intReps = (bsReps == null ? J.adapter.readers.pymol.PyMOLReader.intAt (a, 20) : 0);
-var atomColor = J.adapter.readers.pymol.PyMOLReader.intAt (a, 21);
-var serNo = J.adapter.readers.pymol.PyMOLReader.intAt (a, 22);
-var cartoonType = J.adapter.readers.pymol.PyMOLReader.intAt (a, 23);
-var flags = J.adapter.readers.pymol.PyMOLReader.intAt (a, 24);
-var bonded = (J.adapter.readers.pymol.PyMOLReader.intAt (a, 25) != 0);
-var uniqueID = (a.size () > 40 && J.adapter.readers.pymol.PyMOLReader.intAt (a, 40) == 1 ? J.adapter.readers.pymol.PyMOLReader.intAt (a, 32) : -1);
+atom.partialCharge = partialCharge;
 atom.vib = JU.V3.new3 (uniqueID, cartoonType, NaN);
-if (a.size () > 46) {
-var data = J.adapter.readers.pymol.PyMOLScene.floatsAt (a, 41,  Clazz.newFloatArray (8, 0), 6);
-if (data[0] != 0) this.asc.setAnisoBorU (atom, data, 12);
-}this.pymolScene.setAtomColor (atomColor);
+if (anisou != null && anisou[0] != 0) this.asc.setAnisoBorU (atom, anisou, 12);
+this.pymolScene.setAtomColor (atomColor);
 this.processAtom2 (atom, serNo, x, y, z, formalCharge);
 if (!bonded) this.pymolScene.bsNonbonded.set (this.$ac);
 if (!label.equals (" ")) {
 this.pymolScene.bsLabeled.set (this.$ac);
-var labelOffset = this.listAt (labelPositions, apt);
-this.pymolScene.addLabel (this.$ac, uniqueID, atomColor, labelOffset, label);
+var labelPos =  Clazz.newFloatArray (7, 0);
+if (labelArray != null) {
+for (var i = 0; i < 7; i++) labelPos[i] = labelArray[apt * 7 + i];
+
+} else {
+var labelOffset = J.adapter.readers.pymol.PyMOLReader.listAt (labelPositions, apt);
+if (labelOffset != null) {
+for (var i = 0; i < 7; i++) labelPos[i] = J.adapter.readers.pymol.PyMOLReader.floatAt (labelOffset, i);
+
+}}this.pymolScene.addLabel (this.$ac, uniqueID, atomColor, labelPos, label);
 }if (this.isHidden) this.pymolScene.bsHidden.set (this.$ac);
 if (isNucleic) this.pymolScene.bsNucleic.set (this.$ac);
 for (var i = 0; i < 21; i++) if (bsReps == null ? ((intReps & (1 << i)) != 0) : bsReps.get (i)) this.reps[i].set (this.$ac);
@@ -616,7 +823,33 @@ if (atom.elementSymbol.equals ("H")) this.pymolScene.bsHydrogen.set (this.$ac);
 if ((flags & J.adapter.readers.pymol.PyMOL.FLAG_NOSURFACE) != 0) this.pymolScene.bsNoSurface.set (this.$ac);
 this.atomMap[apt] = this.$ac++;
 return null;
-}, "JU.Lst,~N,~N,JU.Lst,JU.Lst,JU.BS,~N");
+}, "JU.Lst,~N,~A,~A,~A,~N,JU.Lst,~A,JU.Lst,~A,JU.BS,~N");
+Clazz.defineMethod (c$, "atomBool", 
+ function (atomArray, pt, offset, mask) {
+return ((atomArray[pt + offset] & mask) != 0);
+}, "~A,~N,~N,~N");
+Clazz.defineMethod (c$, "atomFloat", 
+ function (atomArray, pt, offset) {
+try {
+return JU.BC.bytesToFloat (atomArray, pt + offset, false);
+} catch (e) {
+if (Clazz.exceptionOf (e, Exception)) {
+return 0;
+} else {
+throw e;
+}
+}
+}, "~A,~N,~N");
+Clazz.defineMethod (c$, "atomStr", 
+ function (atomArray, pt, offset, lexStr) {
+if (offset < 0) return lexStr[JU.BC.bytesToInt (atomArray, pt - offset, false)];
+var s = this.getCStr (atomArray, pt + offset);
+return (s.length == 0 ? " " : s);
+}, "~A,~N,~N,~A");
+Clazz.defineMethod (c$, "atomInt", 
+ function (atomArray, pt, offset) {
+return JU.BC.bytesToInt (atomArray, pt + offset, false);
+}, "~A,~N,~N");
 Clazz.defineMethod (c$, "addBonds", 
  function (bonds) {
 var n = bonds.size ();
@@ -707,7 +940,7 @@ obj.addLast (isosurfaceName);
 this.pymolScene.mepList += ";" + isosurfaceName + ";";
 } else {
 tok = 1073742018;
-mapName = J.adapter.readers.pymol.PyMOLReader.stringAt (this.listAt (this.listAt (obj, 2), 0), 1);
+mapName = J.adapter.readers.pymol.PyMOLReader.stringAt (J.adapter.readers.pymol.PyMOLReader.sublistAt (obj, [2, 0]), 1);
 }var surface = this.volumeData.get (mapName);
 if (surface == null) continue;
 obj.addLast (mapName);
@@ -726,11 +959,11 @@ this.appendLoadNote (s);
 Clazz.defineMethod (c$, "processSelectionsAndScenes", 
  function (map) {
 if (!this.pymolScene.needSelections ()) return;
-var htObjNames = J.adapter.readers.pymol.PyMOLScene.listToMap (J.adapter.readers.pymol.PyMOLReader.getMapList (map, "names"));
+var htObjNames = J.adapter.readers.pymol.PyMOLReader.listToMap (J.adapter.readers.pymol.PyMOLReader.getMapList (map, "names"));
 if (this.haveScenes) {
 var scenes = map.get ("scene_dict");
 this.finalizeSceneData ();
-var htSecrets = J.adapter.readers.pymol.PyMOLScene.listToMap (J.adapter.readers.pymol.PyMOLReader.getMapList (map, "selector_secrets"));
+var htSecrets = J.adapter.readers.pymol.PyMOLReader.listToMap (J.adapter.readers.pymol.PyMOLReader.getMapList (map, "selector_secrets"));
 for (var i = 0; i < this.sceneOrder.size (); i++) {
 var name = J.adapter.readers.pymol.PyMOLReader.stringAt (this.sceneOrder, i);
 var thisScene = J.adapter.readers.pymol.PyMOLReader.getMapList (scenes, name);
@@ -762,14 +995,71 @@ lastAtomSet = this.atoms[i].atomSetIndex;
 this.pymolScene.setAtomInfo (uniqueIDs, cartoonTypes, sequenceNumbers, newChain, radii);
 });
 c$.intAt = Clazz.defineMethod (c$, "intAt", 
- function (list, i) {
+function (list, i) {
 return (list == null ? -1 : (list.get (i)).intValue ());
 }, "JU.Lst,~N");
-c$.stringAt = Clazz.defineMethod (c$, "stringAt", 
- function (list, i) {
-var s = list.get (i).toString ();
-return (s.length == 0 ? " " : s);
+c$.pointAt = Clazz.defineMethod (c$, "pointAt", 
+function (list, i, pt) {
+pt.set (J.adapter.readers.pymol.PyMOLReader.floatAt (list, i++), J.adapter.readers.pymol.PyMOLReader.floatAt (list, i++), J.adapter.readers.pymol.PyMOLReader.floatAt (list, i));
+return pt;
+}, "JU.Lst,~N,JU.P3");
+c$.floatsAt = Clazz.defineMethod (c$, "floatsAt", 
+function (a, pt, data, len) {
+if (a == null) return null;
+for (var i = 0; i < len; i++) data[i] = J.adapter.readers.pymol.PyMOLReader.floatAt (a, pt++);
+
+return data;
+}, "JU.Lst,~N,~A,~N");
+c$.floatAt = Clazz.defineMethod (c$, "floatAt", 
+function (list, i) {
+return (list == null || i >= list.size () ? 0 : (list.get (i)).floatValue ());
 }, "JU.Lst,~N");
+c$.listAt = Clazz.defineMethod (c$, "listAt", 
+function (list, i) {
+if (list == null || i >= list.size ()) return null;
+var o = list.get (i);
+return (Clazz.instanceOf (o, JU.Lst) ? o : null);
+}, "JU.Lst,~N");
+c$.sublistAt = Clazz.defineMethod (c$, "sublistAt", 
+function (mesh, pt) {
+for (var i = 0; i < pt.length; i++) mesh = mesh.get (pt[i]);
+
+return mesh;
+}, "JU.Lst,~A");
+c$.listToMap = Clazz.defineMethod (c$, "listToMap", 
+function (list) {
+var map =  new java.util.Hashtable ();
+for (var i = list.size (); --i >= 0; ) {
+var item = J.adapter.readers.pymol.PyMOLReader.listAt (list, i);
+if (item != null && item.size () > 0) map.put (J.adapter.readers.pymol.PyMOLReader.stringAt (item, 0), item);
+}
+return map;
+}, "JU.Lst");
+c$.stringAt = Clazz.defineMethod (c$, "stringAt", 
+function (list, i) {
+var a = list.get (i);
+return (a.length == 0 ? " " : J.adapter.readers.pymol.PyMOLReader.bytesToString (a));
+}, "JU.Lst,~N");
+c$.bytesToString = Clazz.defineMethod (c$, "bytesToString", 
+function (object) {
+try {
+return  String.instantialize (object, "UTF-8");
+} catch (e) {
+if (Clazz.exceptionOf (e, Exception)) {
+return object.toString ();
+} else {
+throw e;
+}
+}
+}, "~O");
+c$.colorSettingClamped = Clazz.defineMethod (c$, "colorSettingClamped", 
+function (c, ptTemp) {
+return J.adapter.readers.pymol.PyMOLReader.getColorPt (c.get (c.size () < 6 || J.adapter.readers.pymol.PyMOLReader.intAt (c, 4) == 0 ? 2 : 5), ptTemp);
+}, "JU.Lst,JU.P3");
+c$.getColorPt = Clazz.defineMethod (c$, "getColorPt", 
+function (o, ptTemp) {
+return (o == null ? 0 : Clazz.instanceOf (o, Integer) ? (o).intValue () : JU.CU.colorPtToFFRGB (J.adapter.readers.pymol.PyMOLReader.pointAt (o, 0, ptTemp)));
+}, "~O,JU.P3");
 c$.getMapList = Clazz.defineMethod (c$, "getMapList", 
  function (map, key) {
 return map.get (key);
@@ -784,14 +1074,6 @@ if (J.adapter.readers.pymol.PyMOLReader.intAt (list, i) == 1) bsReps.set (i);
 }
 return bsReps;
 }, "JU.Lst");
-Clazz.defineMethod (c$, "floatAt", 
- function (a, i) {
-return J.adapter.readers.pymol.PyMOLScene.floatAt (a, i);
-}, "JU.Lst,~N");
-Clazz.defineMethod (c$, "listAt", 
- function (list, i) {
-return J.adapter.readers.pymol.PyMOLScene.listAt (list, i);
-}, "JU.Lst,~N");
 Clazz.overrideMethod (c$, "getUniqueID", 
 function (iAtom) {
 return Clazz.floatToInt (this.atoms[iAtom].vib.x);
