@@ -142,29 +142,47 @@ if (dbObj == null) return  new JU.BS ();
 var doCache = !key.contains ("NOCACHE");
 if (!doCache) {
 key = JU.PT.rep (key, "NOCACHE", "").trim ();
-}var bs = (doCache ? annotationCache.get (key) : null);
+}System.out.println ("testing DSSR1");
+var bs = null;
 if (bs != null) return bs;
 bs =  new JU.BS ();
 if (doCache) annotationCache.put (key, bs);
 try {
-var pt = key.toLowerCase ().indexOf (" where ");
+key = JU.PT.rep (key, "[where", "[select * where");
+key = JU.PT.rep (key, "[WHERE", "[select * where");
+var ext = "";
+var n = -2147483648;
+var pt = key.toLowerCase ().indexOf ("[select");
+if (pt >= 0) {
+ext = key.substring (pt);
+key = key.substring (0, pt);
+pt = ext.lastIndexOf ("]..");
+if (pt >= 0 && (n = JU.PT.parseInt (ext.substring (pt + 3))) != -2147483648) ext = ext.substring (0, pt + 1);
+}pt = key.toLowerCase ().indexOf (" where ");
 if (pt < 0) {
 key = key.toLowerCase ();
+pt = (n == -2147483648 ? key.lastIndexOf (".") : -1);
+if (pt >= 0 && (n = JU.PT.parseInt (key.substring (pt + 1))) != -2147483648) key = key.substring (0, pt);
 pt = "..bulges.nts_long..coaxstacks.stems.pairs.nt*..hairpins.nts_long..hbonds.atom1_id;atom2_id..helices.pairs.nt*..iloops.nts_long..isocanonpairs.nt*..junctions.nts_long..kissingloops.hairpins.nts_long..multiplets.nts_long..nonstack.nts_long..nts.nt_id..pairs.nt*..sssegments.nts_long..stacks.nts_long..stems.pairs.nt*..".indexOf (".." + key) + 2;
 var len = key.length;
 if (pt < 2) return bs;
 while (pt >= 2 && len > 0) {
-if ("..bulges.nts_long..coaxstacks.stems.pairs.nt*..hairpins.nts_long..hbonds.atom1_id;atom2_id..helices.pairs.nt*..iloops.nts_long..isocanonpairs.nt*..junctions.nts_long..kissingloops.hairpins.nts_long..multiplets.nts_long..nonstack.nts_long..nts.nt_id..pairs.nt*..sssegments.nts_long..stacks.nts_long..stems.pairs.nt*..".substring (pt + len, pt + len + 2).equals ("..")) key = "[select (" + key + ")]";
-dbObj = vwr.extractProperty (dbObj, key, -1);
-pt += len + 1;
+if (key.indexOf (".") < 0 && "..bulges.nts_long..coaxstacks.stems.pairs.nt*..hairpins.nts_long..hbonds.atom1_id;atom2_id..helices.pairs.nt*..iloops.nts_long..isocanonpairs.nt*..junctions.nts_long..kissingloops.hairpins.nts_long..multiplets.nts_long..nonstack.nts_long..nts.nt_id..pairs.nt*..sssegments.nts_long..stacks.nts_long..stems.pairs.nt*..".substring (pt + len, pt + len + 2).equals ("..")) {
+key = "[select (" + key + ")]";
+}dbObj = vwr.extractProperty (dbObj, key, -1);
+if (ext != null) {
+dbObj = vwr.extractProperty (dbObj, ext, -1);
+ext = null;
+}pt += len + 1;
 var pt1 = "..bulges.nts_long..coaxstacks.stems.pairs.nt*..hairpins.nts_long..hbonds.atom1_id;atom2_id..helices.pairs.nt*..iloops.nts_long..isocanonpairs.nt*..junctions.nts_long..kissingloops.hairpins.nts_long..multiplets.nts_long..nonstack.nts_long..nts.nt_id..pairs.nt*..sssegments.nts_long..stacks.nts_long..stems.pairs.nt*..".indexOf (".", pt);
 key = "..bulges.nts_long..coaxstacks.stems.pairs.nt*..hairpins.nts_long..hbonds.atom1_id;atom2_id..helices.pairs.nt*..iloops.nts_long..isocanonpairs.nt*..junctions.nts_long..kissingloops.hairpins.nts_long..multiplets.nts_long..nonstack.nts_long..nts.nt_id..pairs.nt*..sssegments.nts_long..stacks.nts_long..stems.pairs.nt*..".substring (pt, pt1);
 len = key.length;
 }
 } else {
-key = key.substring (0, pt).trim () + "[select * " + key.substring (pt + 1) + "]";
+key = key.substring (0, pt).trim () + "[select * " + key.substring (pt + 1) + "]" + ext;
 dbObj = vwr.extractProperty (dbObj, key, -1);
-}bs.or (vwr.ms.getAtoms (1086324744, dbObj.toString ()));
+}if (n != -2147483648 && Clazz.instanceOf (dbObj, JU.Lst)) dbObj = (dbObj).get (n);
+bs.or (vwr.ms.getAtoms (1086324744, dbObj.toString ()));
 bs.and (bsModel);
 } catch (e) {
 if (Clazz.exceptionOf (e, Exception)) {
