@@ -1,5 +1,5 @@
 Clazz.declarePackage ("J.g3d");
-Clazz.load (null, "J.g3d.CylinderRenderer", ["JU.AU", "$.P3"], function () {
+Clazz.load (["JU.P3i"], "J.g3d.CylinderRenderer", ["JU.AU"], function () {
 c$ = Clazz.decorateAsClass (function () {
 this.g3d = null;
 this.line3d = null;
@@ -49,6 +49,8 @@ Clazz.instantialize (this, arguments);
 Clazz.prepareFields (c$, function () {
 this.xyztRaster =  Clazz.newArray (-1, [ Clazz.newFloatArray (32, 0),  Clazz.newFloatArray (32, 0),  Clazz.newFloatArray (32, 0),  Clazz.newFloatArray (32, 0)]);
 this.xyzfRaster =  Clazz.newArray (-1, [ Clazz.newIntArray (32, 0),  Clazz.newIntArray (32, 0),  Clazz.newIntArray (32, 0),  Clazz.newIntArray (32, 0)]);
+this.ptA0 =  new JU.P3i ();
+this.ptB0 =  new JU.P3i ();
 });
 Clazz.makeConstructor (c$, 
 function (g3d) {
@@ -120,19 +122,16 @@ Clazz.defineMethod (c$, "renderBits",
 function (colixA, colixB, screen, endcaps, diameter, ptA, ptB) {
 var g = this.g3d;
 if (diameter == 0 || diameter == 1) {
-this.line3d.plotLineBits (g.getColorArgbOrGray (colixA), g.getColorArgbOrGray (colixB), ptA, ptB);
+this.line3d.plotLineBits (g.getColorArgbOrGray (colixA), g.getColorArgbOrGray (colixB), ptA, ptB, 0, 0, false);
 return;
-}if (this.ptA0 == null) {
-this.ptA0 =  new JU.P3 ();
-this.ptB0 =  new JU.P3 ();
 }this.ptA0.setT (ptA);
 var r = Clazz.doubleToInt (diameter / 2) + 1;
-var ixA = Math.round (ptA.x);
-var iyA = Math.round (ptA.y);
-var izA = Math.round (ptA.z);
-var ixB = Math.round (ptB.x);
-var iyB = Math.round (ptB.y);
-var izB = Math.round (ptB.z);
+var ixA = ptA.x;
+var iyA = ptA.y;
+var izA = ptA.z;
+var ixB = ptB.x;
+var iyB = ptB.y;
+var izB = ptB.z;
 var codeMinA = g.clipCode3 (ixA - r, iyA - r, izA - r);
 var codeMaxA = g.clipCode3 (ixA + r, iyA + r, izA + r);
 var codeMinB = g.clipCode3 (ixB - r, iyB - r, izB - r);
@@ -203,7 +202,7 @@ if (endcaps == 3) this.renderSphericalEndcaps ();
 this.xAf += this.dxBf;
 this.yAf += this.dyBf;
 this.zAf += this.dzBf;
-}, "~N,~N,~N,~N,~N,JU.P3,JU.P3");
+}, "~N,~N,~N,~N,~N,JU.P3i,JU.P3i");
 Clazz.defineMethod (c$, "renderConeOld", 
 function (colix, endcap, diameter, xa, ya, za, xtip, ytip, ztip, doFill, isBarb) {
 this.dxBf = (xtip) - (this.xAf = xa);
