@@ -1,5 +1,5 @@
 Clazz.declarePackage ("J.g3d");
-Clazz.load (["J.g3d.PrecisionRenderer", "java.util.Hashtable"], "J.g3d.LineRenderer", ["java.lang.Float", "JU.BS", "$.P3"], function () {
+Clazz.load (["J.g3d.PrecisionRenderer", "java.util.Hashtable"], "J.g3d.LineRenderer", ["java.lang.Float", "JU.BS"], function () {
 c$ = Clazz.decorateAsClass (function () {
 this.g3d = null;
 this.shader = null;
@@ -9,8 +9,6 @@ this.lineTypeX = false;
 this.nBits = 0;
 this.lineCache = null;
 this.slopeKey = null;
-this.ptAtrimmed = null;
-this.ptBtrimmed = null;
 this.x1t = 0;
 this.y1t = 0;
 this.z1t = 0;
@@ -60,14 +58,15 @@ function () {
 this.lineCache.clear ();
 });
 Clazz.defineMethod (c$, "plotLineOld", 
-function (argbA, argbB, xA, yA, zA, xB, yB, zB, clipped) {
+function (argbA, argbB, xA, yA, zA, xB, yB, zB) {
 this.x1t = xA;
 this.x2t = xB;
 this.y1t = yA;
 this.y2t = yB;
 this.z1t = zA;
 this.z2t = zB;
-if (clipped) switch (this.getTrimmedLineImpl ()) {
+var clipped = true;
+switch (this.getTrimmedLineImpl ()) {
 case 0:
 clipped = false;
 break;
@@ -75,7 +74,7 @@ case 2:
 return;
 }
 this.plotLineClippedOld (argbA, argbB, xA, yA, zA, xB - xA, yB - yA, zB - zA, clipped, 0, 0);
-}, "~N,~N,~N,~N,~N,~N,~N,~N,~B");
+}, "~N,~N,~N,~N,~N,~N,~N,~N");
 Clazz.defineMethod (c$, "plotLineDeltaOld", 
 function (argbA, argbB, xA, yA, zA, dxBA, dyBA, dzBA, clipped) {
 this.x1t = xA;
@@ -205,12 +204,12 @@ p.addPixel (offset, zCurrent, rand8 < 85 ? argbDn : (rand8 > 170 ? argbUp : argb
 }}, "~A,~A,~N,~N,~N,~N,~N,~N,~N,~N,~B");
 Clazz.defineMethod (c$, "plotLineDeltaABits", 
 function (shades1, shades2, shadeIndex, ptA, ptB, screenMask, clipped) {
-var x = Math.round (ptA.x);
-var y = Math.round (ptA.y);
-var z = Math.round (ptA.z);
-var bx = Math.round (ptB.x);
-var by = Math.round (ptB.y);
-var bz = Math.round (ptB.z);
+var x = ptA.x;
+var y = ptA.y;
+var z = ptA.z;
+var bx = ptB.x;
+var by = ptB.y;
+var bz = ptB.z;
 var dx = bx - x;
 var dy = by - y;
 this.x1t = x;
@@ -292,51 +291,100 @@ while (iBits < 0) iBits += this.nBits;
 if (this.lineBits.get (iBits % this.nBits)) {
 offset += yOffsetIncrement;
 }}
-}, "~A,~A,~N,JU.P3,JU.P3,~N,~B");
+}, "~A,~A,~N,JU.P3i,JU.P3i,~N,~B");
 Clazz.defineMethod (c$, "plotLineBits", 
-function (argbA, argbB, ptA, ptB) {
+function (argbA, argbB, ptA, ptB, run, rise, andClip) {
 if (ptA.z <= 1 || ptB.z <= 1) return;
 var clipped = true;
-switch (this.getTrimmedLineP3 (ptA, ptB)) {
-case 2:
-return;
-case 0:
-clipped = false;
-break;
-}
-this.plotLineClippedBits (argbA, argbB, ptA, ptB, clipped, 0, 0);
-}, "~N,~N,JU.P3,JU.P3");
-Clazz.defineMethod (c$, "plotDashedLineBits", 
-function (argb, run, rise, ptA, ptB) {
-if (ptA.z <= 1 || ptB.z <= 1) return;
-var clipped = true;
-switch (this.getTrimmedLineP3 (ptA, ptB)) {
+this.x1t = ptA.x;
+this.y1t = ptA.y;
+this.z1t = ptA.z;
+this.x2t = ptB.x;
+this.y2t = ptB.y;
+this.z2t = ptB.z;
+switch (this.getTrimmedLineImpl ()) {
 case 2:
 return;
 case 0:
 clipped = false;
 break;
 default:
-if (this.ptAtrimmed == null) {
-this.ptAtrimmed =  new JU.P3 ();
-this.ptBtrimmed =  new JU.P3 ();
-}this.ptAtrimmed.set (this.x1t, this.y1t, this.z1t);
-this.ptBtrimmed.set (this.x2t, this.y2t, this.z2t);
-ptA = this.ptAtrimmed;
-ptB = this.ptBtrimmed;
+if (andClip) {
+ptA.set (this.x1t, this.y1t, this.z1t);
+ptB.set (this.x2t, this.y2t, this.z2t);
+}}
+var zbuf = this.g3d.zbuf;
+var width = this.g3d.width;
+var runIndex = 0;
+if (run == 0) {
+rise = 2147483647;
+run = 1;
+}var x = ptA.x;
+var y = ptA.y;
+var z = ptA.z;
+var dx = ptB.x - x;
+var x2 = x + dx;
+var dy = ptB.y - y;
+var y2 = y + dy;
+var offset = y * width + x;
+var offsetMax = this.g3d.bufferSize;
+var argb = argbA;
+var p = this.g3d.pixel;
+if (argb != 0 && !clipped && offset >= 0 && offset < offsetMax && z < zbuf[offset]) p.addPixel (offset, z, argb);
+if (dx == 0 && dy == 0) return;
+var xIncrement = 1;
+var yIncrement = 1;
+var yOffsetIncrement = width;
+if (dx < 0) {
+dx = -dx;
+xIncrement = -1;
+}if (dy < 0) {
+dy = -dy;
+yOffsetIncrement = -width;
+yIncrement = -1;
+}var twoDx = dx + dx;
+var twoDy = dy + dy;
+if (dy <= dx) {
+this.setRastAB (ptA.x, ptA.z, ptB.x, ptB.z);
+var twoDxAccumulatedYError = 0;
+var n1 = Math.abs (x2 - this.x2t) - 1;
+var n2 = Math.abs (x2 - this.x1t) - 1;
+for (var n = dx - 1, nMid = Clazz.doubleToInt (n / 2); --n >= n1; ) {
+if (n == nMid) {
+argb = argbB;
+if (argb == 0) return;
+}offset += xIncrement;
+x += xIncrement;
+twoDxAccumulatedYError += twoDy;
+if (twoDxAccumulatedYError > dx) {
+offset += yOffsetIncrement;
+twoDxAccumulatedYError -= twoDx;
+}if (argb != 0 && n < n2 && offset >= 0 && offset < offsetMax && runIndex < rise) {
+var zCurrent = this.getZCurrent (this.a, this.b, x);
+if (zCurrent < zbuf[offset]) p.addPixel (offset, zCurrent, argb);
+}runIndex = (runIndex + 1) % run;
 }
-this.plotLineClippedBits (argb, argb, ptA, ptB, clipped, run, rise);
-}, "~N,~N,~N,JU.P3,JU.P3");
-Clazz.defineMethod (c$, "getTrimmedLineP3", 
- function (ptA, ptB) {
-this.x1t = Clazz.floatToInt (ptA.x);
-this.y1t = Clazz.floatToInt (ptA.y);
-this.z1t = Clazz.floatToInt (ptA.z);
-this.x2t = Clazz.floatToInt (ptB.x);
-this.y2t = Clazz.floatToInt (ptB.y);
-this.z2t = Clazz.floatToInt (ptB.z);
-return this.getTrimmedLineImpl ();
-}, "JU.P3,JU.P3");
+} else {
+this.setRastAB (ptA.y, ptA.z, ptB.y, ptB.z);
+var twoDyAccumulatedXError = 0;
+var n1 = Math.abs (y2 - this.y2t) - 1;
+var n2 = Math.abs (y2 - this.y1t) - 1;
+for (var n = dy - 1, nMid = Clazz.doubleToInt (n / 2); --n >= n1; ) {
+if (n == nMid) {
+argb = argbB;
+if (argb == 0) return;
+}offset += yOffsetIncrement;
+y += yIncrement;
+twoDyAccumulatedXError += twoDx;
+if (twoDyAccumulatedXError > dy) {
+offset += xIncrement;
+twoDyAccumulatedXError -= twoDy;
+}if (argb != 0 && n < n2 && offset >= 0 && offset < offsetMax && runIndex < rise) {
+var zCurrent = this.getZCurrent (this.a, this.b, y);
+if (zCurrent < zbuf[offset]) p.addPixel (offset, zCurrent, argb);
+}runIndex = (runIndex + 1) % run;
+}
+}}, "~N,~N,JU.P3i,JU.P3i,~N,~N,~B");
 Clazz.defineMethod (c$, "getTrimmedLineImpl", 
  function () {
 var cc1 = this.g3d.clipCode3 (this.x1t, this.y1t, this.z1t);
@@ -480,80 +528,6 @@ if (zCurrent < zbuf[offset]) p.addPixel (offset, zCurrent, argb);
 }runIndex = (runIndex + 1) % run;
 }
 }}, "~N,~N,~N,~N,~N,~N,~N,~N,~B,~N,~N");
-Clazz.defineMethod (c$, "plotLineClippedBits", 
- function (argb1, argb2, ptA, ptB, clipped, run, rise) {
-var zbuf = this.g3d.zbuf;
-var width = this.g3d.width;
-var runIndex = 0;
-if (run == 0) {
-rise = 2147483647;
-run = 1;
-}var x = Clazz.floatToInt (ptA.x);
-var y = Clazz.floatToInt (ptA.y);
-var z = Clazz.floatToInt (ptA.z);
-var dx = Clazz.floatToInt (ptB.x) - x;
-var x2 = x + dx;
-var dy = Clazz.floatToInt (ptB.y) - y;
-var y2 = y + dy;
-var offset = y * width + x;
-var offsetMax = this.g3d.bufferSize;
-var argb = argb1;
-var p = this.g3d.pixel;
-if (argb != 0 && !clipped && offset >= 0 && offset < offsetMax && z < zbuf[offset]) p.addPixel (offset, z, argb);
-if (dx == 0 && dy == 0) return;
-var xIncrement = 1;
-var yIncrement = 1;
-var yOffsetIncrement = width;
-if (dx < 0) {
-dx = -dx;
-xIncrement = -1;
-}if (dy < 0) {
-dy = -dy;
-yOffsetIncrement = -width;
-yIncrement = -1;
-}var twoDx = dx + dx;
-var twoDy = dy + dy;
-if (dy <= dx) {
-this.setRastAB (ptA.x, ptA.z, ptB.x, ptB.z);
-var twoDxAccumulatedYError = 0;
-var n1 = Math.abs (x2 - this.x2t) - 1;
-var n2 = Math.abs (x2 - this.x1t) - 1;
-for (var n = dx - 1, nMid = Clazz.doubleToInt (n / 2); --n >= n1; ) {
-if (n == nMid) {
-argb = argb2;
-if (argb == 0) return;
-}offset += xIncrement;
-x += xIncrement;
-twoDxAccumulatedYError += twoDy;
-if (twoDxAccumulatedYError > dx) {
-offset += yOffsetIncrement;
-twoDxAccumulatedYError -= twoDx;
-}if (argb != 0 && n < n2 && offset >= 0 && offset < offsetMax && runIndex < rise) {
-var zCurrent = this.getZCurrent (this.a, this.b, x);
-if (zCurrent < zbuf[offset]) p.addPixel (offset, zCurrent, argb);
-}runIndex = (runIndex + 1) % run;
-}
-} else {
-this.setRastAB (ptA.y, ptA.z, ptB.y, ptB.z);
-var twoDyAccumulatedXError = 0;
-var n1 = Math.abs (y2 - this.y2t) - 1;
-var n2 = Math.abs (y2 - this.y1t) - 1;
-for (var n = dy - 1, nMid = Clazz.doubleToInt (n / 2); --n >= n1; ) {
-if (n == nMid) {
-argb = argb2;
-if (argb == 0) return;
-}offset += yOffsetIncrement;
-y += yIncrement;
-twoDyAccumulatedXError += twoDx;
-if (twoDyAccumulatedXError > dy) {
-offset += xIncrement;
-twoDyAccumulatedXError -= twoDy;
-}if (argb != 0 && n < n2 && offset >= 0 && offset < offsetMax && runIndex < rise) {
-var zCurrent = this.getZCurrent (this.a, this.b, y);
-if (zCurrent < zbuf[offset]) p.addPixel (offset, zCurrent, argb);
-}runIndex = (runIndex + 1) % run;
-}
-}}, "~N,~N,JU.P3,JU.P3,~B,~N,~N");
 Clazz.defineStatics (c$,
 "VISIBILITY_UNCLIPPED", 0,
 "VISIBILITY_CLIPPED", 1,
