@@ -52,7 +52,8 @@ if (!this.htModels.containsKey (this.strID)) this.htModels.put (this.strID,  new
 this.thisModel = this.htModels.get (this.strID);
 this.$moNumber = (!this.thisModel.containsKey ("moNumber") ? 0 : (this.thisModel.get ("moNumber")).intValue ());
 this.$moLinearCombination = this.thisModel.get ("moLinearCombination");
-this.moSquareData = this.moSquareLinear = null;
+this.moSquareData = (this.$moLinearCombination != null ? null : this.thisModel.get ("moSquareData"));
+this.moSquareLinear = (this.$moLinearCombination == null ? null : this.thisModel.get ("moSquareLinear"));
 return;
 }if ("slab" === propertyName) {
 if (Clazz.instanceOf (value, Integer)) {
@@ -76,12 +77,14 @@ return;
 this.thisModel.put ("moScale", value);
 return;
 }if ("squareData" === propertyName) {
-this.thisModel.put ("moSquareData", Boolean.TRUE);
-this.moSquareData = Boolean.TRUE;
+if (value === Boolean.TRUE) this.thisModel.put ("moSquareData", Boolean.TRUE);
+ else this.thisModel.remove ("moSquareData");
+this.moSquareData = value;
 return;
 }if ("squareLinear" === propertyName) {
-this.thisModel.put ("moSquareLinear", Boolean.TRUE);
-this.moSquareLinear = Boolean.TRUE;
+if (value === Boolean.TRUE) this.thisModel.put ("moSquareLinear", Boolean.TRUE);
+ else this.thisModel.remove ("moSquareLinear");
+this.moSquareLinear = value;
 return;
 }if ("cutoffPositive" === propertyName) {
 this.thisModel.put ("moCutoff", value);
@@ -213,6 +216,8 @@ s = "\n" + s;
 }return this.getMoInfo (-1) + s;
 }if (propertyName === "moNumber") return Integer.$valueOf (this.$moNumber);
 if (propertyName === "moLinearCombination") return this.$moLinearCombination;
+if (propertyName === "moSquareData") return this.moSquareData;
+if (propertyName === "moSquareLinear") return this.moSquareLinear;
 if (propertyName === "showMO") {
 var str =  new JU.SB ();
 var mos = (this.sg.params.moData.get ("mos"));
@@ -315,8 +320,10 @@ if (this.moResolution != null) this.setPropI ("resolution", this.moResolution, n
 if (this.moPlane != null) {
 this.setPropI ("plane", this.moPlane, null);
 if (this.moCutoff != null) {
-this.setPropI ("red", Float.$valueOf (-this.moCutoff.floatValue ()), null);
-this.setPropI ("blue", this.moCutoff, null);
+var max = this.moCutoff.floatValue ();
+if (this.moSquareData === Boolean.TRUE || this.moSquareLinear === Boolean.TRUE) max = max * max;
+this.setPropI ("red", Float.$valueOf (-max), null);
+this.setPropI ("blue", Float.$valueOf (max), null);
 }} else {
 if (this.moCutoff != null) this.setPropI ((this.moIsPositiveOnly ? "cutoffPositive" : "cutoff"), this.moCutoff, null);
 if (this.moColorNeg != null) this.setPropI ("colorRGB", this.moColorNeg, null);
