@@ -42,6 +42,7 @@ this.bfactor100s = null;
 this.partialCharges = null;
 this.bondingRadii = null;
 this.hydrophobicities = null;
+this.bsPartialCharges = null;
 this.aaRet = null;
 if (!Clazz.isClassDefined ("JM.AtomCollection.AtomSorter")) {
 JM.AtomCollection.$AtomCollection$AtomSorter$ ();
@@ -74,6 +75,7 @@ this.vibrations = null;
 this.occupancies = null;
 this.bfactor100s = null;
 this.partialCharges = null;
+this.bsPartialCharges = null;
 this.bondingRadii = null;
 this.atomTensors = null;
 });
@@ -90,6 +92,7 @@ this.occupancies = mergeModelSet.occupancies;
 this.bfactor100s = mergeModelSet.bfactor100s;
 this.bondingRadii = mergeModelSet.bondingRadii;
 this.partialCharges = mergeModelSet.partialCharges;
+this.bsPartialCharges = mergeModelSet.bsPartialCharges;
 this.atomTensors = mergeModelSet.atomTensors;
 this.atomTensorList = mergeModelSet.atomTensorList;
 this.bsModulated = mergeModelSet.bsModulated;
@@ -630,11 +633,13 @@ if (doTaint) this.taintAtom (atomIndex, 7);
 }, "~N,~N,~B");
 Clazz.defineMethod (c$, "setPartialCharge", 
 function (atomIndex, partialCharge, doTaint) {
-if (Float.isNaN (partialCharge) || doTaint && partialCharge == this.at[atomIndex].getPartialCharge ()) return;
+if (Float.isNaN (partialCharge)) return;
 if (this.partialCharges == null) {
+this.bsPartialCharges =  new JU.BS ();
 if (partialCharge == 0) return;
 this.partialCharges =  Clazz.newFloatArray (this.at.length, 0);
-}this.partialCharges[atomIndex] = partialCharge;
+}this.bsPartialCharges.set (atomIndex);
+this.partialCharges[atomIndex] = partialCharge;
 if (doTaint) this.taintAtom (atomIndex, 8);
 }, "~N,~N,~B");
 Clazz.defineMethod (c$, "setBondingRadius", 
@@ -1823,6 +1828,7 @@ this.bfactor100s = JU.AU.deleteElements (this.bfactor100s, firstAtomIndex, nAtom
 this.hasBfactorRange = false;
 this.occupancies = JU.AU.deleteElements (this.occupancies, firstAtomIndex, nAtoms);
 this.partialCharges = JU.AU.deleteElements (this.partialCharges, firstAtomIndex, nAtoms);
+this.bsPartialCharges = JU.BSUtil.deleteBits (this.bsPartialCharges, bsAtoms);
 this.atomTensorList = JU.AU.deleteElements (this.atomTensorList, firstAtomIndex, nAtoms);
 this.vibrations = JU.AU.deleteElements (this.vibrations, firstAtomIndex, nAtoms);
 this.nSurfaceAtoms = 0;
