@@ -9,7 +9,9 @@ this.primitives = null;
 this.nPrimitives = 0;
 this.bondsOr = null;
 this.nBondsOr = 0;
-this.isRingBond = false;
+this.isConnection = false;
+this.atropType = null;
+this.isChain = false;
 Clazz.instantialize (this, arguments);
 }, JS, "SmilesBond", JU.Edge);
 c$.getBondOrderString = Clazz.defineMethod (c$, "getBondOrderString", 
@@ -19,10 +21,44 @@ case 2:
 return "=";
 case 3:
 return "#";
+case 4:
+return "$";
 default:
 return "";
 }
 }, "~N");
+c$.getBondTypeFromCode = Clazz.defineMethod (c$, "getBondTypeFromCode", 
+function (code) {
+switch (code) {
+case '.':
+return 0;
+case '-':
+return 1;
+case '=':
+return 2;
+case '#':
+return 3;
+case '$':
+return 4;
+case ':':
+return 17;
+case '/':
+return 1025;
+case '\\':
+return 1041;
+case '^':
+return 65537;
+case '`':
+return 65538;
+case '@':
+return 65;
+case '~':
+return 81;
+case '+':
+return 96;
+}
+return -1;
+}, "~S");
 Clazz.defineMethod (c$, "set", 
 function (bond) {
 this.order = bond.order;
@@ -32,6 +68,18 @@ this.nPrimitives = bond.nPrimitives;
 this.bondsOr = bond.bondsOr;
 this.nBondsOr = bond.nBondsOr;
 }, "JS.SmilesBond");
+Clazz.defineMethod (c$, "setAtropType", 
+function (nn) {
+this.atropType =  Clazz.newIntArray (-1, [Clazz.doubleToInt (nn / 10) - 1, nn % 10 - 1]);
+}, "~N");
+Clazz.defineMethod (c$, "setPrimitive", 
+function (i) {
+var p = this.primitives[i];
+this.order = p.order;
+this.isNot = p.isNot;
+this.atropType = p.atropType;
+return p;
+}, "~N");
 Clazz.defineMethod (c$, "addBondOr", 
 function () {
 if (this.bondsOr == null) this.bondsOr =  new Array (2);
@@ -72,59 +120,42 @@ this.order = bondType;
 this.isNot = isNot;
 }, "~N,~B");
 Clazz.defineMethod (c$, "set2a", 
-function (atom1, atom2) {
-if (atom1 != null) {
-this.atom1 = atom1;
-atom1.addBond (this);
-}if (atom2 != null) {
-this.atom2 = atom2;
-atom2.isFirst = false;
-atom2.addBond (this);
+function (a1, a2) {
+if (a1 != null) {
+this.atom1 = a1;
+a1.addBond (this);
+}if (a2 != null) {
+this.atom2 = a2;
+if (a2.isBioAtomWild && this.atom1.isBioAtomWild) this.order = 96;
+a2.isFirst = false;
+a2.addBond (this);
 }}, "JS.SmilesAtom,JS.SmilesAtom");
 Clazz.defineMethod (c$, "setAtom2", 
-function (atom) {
+function (atom, molecule) {
 this.atom2 = atom;
 if (this.atom2 != null) {
 atom.addBond (this);
-this.isRingBond = true;
-}}, "JS.SmilesAtom");
+this.isConnection = true;
+}}, "JS.SmilesAtom,JS.SmilesSearch");
+Clazz.defineMethod (c$, "isFromPreviousTo", 
+function (atom) {
+return (!this.isConnection && this.atom2 === atom);
+}, "JS.SmilesAtom");
 c$.isBondType = Clazz.defineMethod (c$, "isBondType", 
 function (ch, isSearch, isBioSequence) {
-if ("-=#:/\\.+!,&;@~^'".indexOf (ch) < 0) return false;
-if (!isSearch && "-=#:/\\.~^'".indexOf (ch) < 0) throw  new JS.InvalidSmilesException ("SMARTS bond type " + ch + " not allowed in SMILES");
-if (isBioSequence && ch == '~') return false;
-return true;
-}, "~S,~B,~B");
-c$.getBondTypeFromCode = Clazz.defineMethod (c$, "getBondTypeFromCode", 
-function (code) {
-switch (code) {
-case '.':
-return 0;
-case '-':
-return 1;
-case '=':
-return 2;
-case '#':
-return 3;
-case ':':
-return 17;
-case '/':
-return 257;
-case '\\':
-return 513;
-case '^':
-return 769;
-case '\'':
-return 1025;
-case '@':
-return 65;
+if (ch == '>') return 1;
+if ("-=#$:/\\.~^`+!,&;@".indexOf (ch) < 0) return 0;
+if (!isSearch && "-=#$:/\\.~^`".indexOf (ch) < 0) throw  new JS.InvalidSmilesException ("SMARTS bond type " + ch + " not allowed in SMILES");
+switch (ch) {
 case '~':
-return 81;
-case '+':
-return 96;
-}
+return (isBioSequence ? 0 : 1);
+case '^':
+case '`':
 return -1;
-}, "~S");
+default:
+return 1;
+}
+}, "~S,~B,~B");
 Clazz.defineMethod (c$, "getBondType", 
 function () {
 return this.order;
@@ -167,34 +198,28 @@ var a = this.atom1;
 this.atom1 = this.atom2;
 this.atom2 = a;
 switch (this.order) {
-case 769:
-this.order = 1025;
+case 65537:
+this.order = 65538;
+break;
+case 65538:
+this.order = 65537;
 break;
 case 1025:
-this.order = 769;
+this.order = 1041;
 break;
-case 257:
-this.order = 513;
-break;
-case 513:
-this.order = 257;
+case 1041:
+this.order = 1025;
 break;
 }
 });
 Clazz.defineStatics (c$,
 "TYPE_UNKNOWN", -1,
 "TYPE_NONE", 0,
-"TYPE_SINGLE", 1,
-"TYPE_DOUBLE", 2,
-"TYPE_TRIPLE", 3,
 "TYPE_AROMATIC", 0x11,
-"TYPE_DIRECTIONAL_1", 0x101,
-"TYPE_DIRECTIONAL_2", 0x201,
-"TYPE_ATROPISOMER_1", 0x301,
-"TYPE_ATROPISOMER_2", 0x401,
 "TYPE_RING", 0x41,
 "TYPE_ANY", 0x51,
 "TYPE_BIO_SEQUENCE", 0x60,
 "TYPE_BIO_CROSSLINK", 0x70,
-"TYPE_MULTIPLE", 999);
+"ALL_BONDS", "-=#$:/\\.~^`+!,&;@",
+"SMILES_BONDS", "-=#$:/\\.~^`");
 });
