@@ -38,7 +38,7 @@ if (m.bsSlabDisplay != null && (m.polygonCount0 != 0 || m.vertexCount0 != 0)) {
 m.pc = m.polygonCount0;
 m.vc = m.vertexCount0;
 m.polygonCount0 = m.vertexCount0 = 0;
-m.normixCount = (m.isTriangleSet ? m.pc : m.vc);
+m.normixCount = (m.isDrawPolygon ? m.pc : m.vc);
 m.bsSlabDisplay.setBits (0, (m.pc == 0 ? m.vc : m.pc));
 m.slabOptions =  new JU.SB ().append (m.meshType + " slab none");
 m.bsSlabGhost = null;
